@@ -42,14 +42,14 @@ from open_webui.utils.auth import decode_token
 from open_webui.socket.utils import RedisDict, RedisLock, YdocManager
 from open_webui.tasks import create_task, stop_item_tasks
 from open_webui.utils.redis import get_redis_connection
-from open_webui.utils.access_control import has_access, get_users_with_access
+from open_webui.utils.access_control import has_permission
+from open_webui.models.access_grants import AccessGrants
 
 
 from open_webui.env import (
     GLOBAL_LOG_LEVEL,
 )
 
-
 logging.basicConfig(stream=sys.stdout, level=GLOBAL_LOG_LEVEL)
 log = logging.getLogger(__name__)
 
@@ -99,6 +99,7 @@ else:
 
 # Timeout duration in seconds
 TIMEOUT_DURATION = 3
+SESSION_POOL_TIMEOUT = 120  # seconds without heartbeat before session is reaped
 
 # Dictionary to maintain the user pool
 
@@ -147,6 +148,17 @@ if WEBSOCKET_MANAGER == "redis":
     aquire_func = clean_up_lock.aquire_lock
     renew_func = clean_up_lock.renew_lock
     release_func = clean_up_lock.release_lock
+
+    session_cleanup_lock = RedisLock(
+        redis_url=WEBSOCKET_REDIS_URL,
+        lock_name=f"{REDIS_KEY_PREFIX}:session_cleanup_lock",
+        timeout_secs=WEBSOCKET_REDIS_LOCK_TIMEOUT,
+        redis_sentinels=redis_sentinels,
+        redis_cluster=WEBSOCKET_REDIS_CLUSTER,
+    )
+    session_aquire_func = session_cleanup_lock.aquire_lock
+    session_renew_func = session_cleanup_lock.renew_lock
+    session_release_func = session_cleanup_lock.release_lock
 else:
     MODELS = {}
 
@@ -154,6 +166,7 @@ else:
     USAGE_POOL = {}
 
     aquire_func = release_func = renew_func = lambda: True
+    session_aquire_func = session_release_func = session_renew_func = lambda: True
 
 
 YDOC_MANAGER = YdocManager(
@@ -162,6 +175,31 @@ YDOC_MANAGER = YdocManager(
 )
 
 
+async def periodic_session_pool_cleanup():
+    """Reap orphaned SESSION_POOL entries that missed heartbeats (e.g. crashed instance)."""
+    if not session_aquire_func():
+        log.debug("Session cleanup lock held by another node. Skipping.")
+        return
+
+    try:
+        while True:
+            if not session_renew_func():
+                log.error("Unable to renew session cleanup lock. Exiting.")
+                return
+
+            now = int(time.time())
+            for sid in list(SESSION_POOL.keys()):
+                entry = SESSION_POOL.get(sid)
+                if entry and now - entry.get("last_seen_at", 0) > SESSION_POOL_TIMEOUT:
+                    log.warning(
+                        f"Reaping orphaned session {sid} (user {entry.get('id')})"
+                    )
+                    del SESSION_POOL[sid]
+            await asyncio.sleep(SESSION_POOL_TIMEOUT)
+    finally:
+        session_release_func()
+
+
 async def periodic_usage_pool_cleanup():
     max_retries = 2
     retry_delay = random.uniform(
@@ -246,7 +284,13 @@ def get_user_ids_from_room(room):
     active_session_ids = get_session_ids_from_room(room)
 
     active_user_ids = list(
-        set([SESSION_POOL.get(session_id)["id"] for session_id in active_session_ids])
+        set(
+            [
+                SESSION_POOL.get(session_id)["id"]
+                for session_id in active_session_ids
+                if SESSION_POOL.get(session_id) is not None
+            ]
+        )
     )
     return active_user_ids
 
@@ -307,9 +351,18 @@ async def connect(sid, environ, auth):
             user = Users.get_user_by_id(data["id"])
 
         if user:
-            SESSION_POOL[sid] = user.model_dump(
-                exclude=["date_of_birth", "bio", "gender"]
-            )
+            SESSION_POOL[sid] = {
+                **user.model_dump(
+                    exclude=[
+                        "profile_image_url",
+                        "profile_banner_image_url",
+                        "date_of_birth",
+                        "bio",
+                        "gender",
+                    ]
+                ),
+                "last_seen_at": int(time.time()),
+            }
             await sio.enter_room(sid, f"user:{user.id}")
 
 
@@ -328,23 +381,27 @@ async def user_join(sid, data):
     if not user:
         return
 
-    SESSION_POOL[sid] = user.model_dump(
-        exclude=[
-            "profile_image_url",
-            "profile_banner_image_url",
-            "date_of_birth",
-            "bio",
-            "gender",
-        ]
-    )
+    SESSION_POOL[sid] = {
+        **user.model_dump(
+            exclude=[
+                "profile_image_url",
+                "profile_banner_image_url",
+                "date_of_birth",
+                "bio",
+                "gender",
+            ]
+        ),
+        "last_seen_at": int(time.time()),
+    }
 
     await sio.enter_room(sid, f"user:{user.id}")
 
-    # Join all the channels
-    channels = Channels.get_channels_by_user_id(user.id)
-    log.debug(f"{channels=}")
-    for channel in channels:
-        await sio.enter_room(sid, f"channel:{channel.id}")
+    # Join all the channels only if user has channels permission
+    if user.role == "admin" or has_permission(user.id, "features.channels"):
+        channels = Channels.get_channels_by_user_id(user.id)
+        log.debug(f"{channels=}")
+        for channel in channels:
+            await sio.enter_room(sid, f"channel:{channel.id}")
 
     return {"id": user.id, "name": user.name}
 
@@ -353,6 +410,7 @@ async def user_join(sid, data):
 async def heartbeat(sid, data):
     user = SESSION_POOL.get(sid)
     if user:
+        SESSION_POOL[sid] = {**user, "last_seen_at": int(time.time())}
         Users.update_last_active_by_id(user["id"])
 
 
@@ -370,11 +428,12 @@ async def join_channel(sid, data):
     if not user:
         return
 
-    # Join all the channels
-    channels = Channels.get_channels_by_user_id(user.id)
-    log.debug(f"{channels=}")
-    for channel in channels:
-        await sio.enter_room(sid, f"channel:{channel.id}")
+    # Join all the channels only if user has channels permission
+    if user.role == "admin" or has_permission(user.id, "features.channels"):
+        channels = Channels.get_channels_by_user_id(user.id)
+        log.debug(f"{channels=}")
+        for channel in channels:
+            await sio.enter_room(sid, f"channel:{channel.id}")
 
 
 @sio.on("join-note")
@@ -399,7 +458,12 @@ async def join_note(sid, data):
     if (
         user.role != "admin"
         and user.id != note.user_id
-        and not has_access(user.id, type="read", access_control=note.access_control)
+        and not AccessGrants.has_access(
+            user_id=user.id,
+            resource_type="note",
+            resource_id=note.id,
+            permission="read",
+        )
     ):
         log.error(f"User {user.id} does not have access to note {data['note_id']}")
         return
@@ -461,8 +525,11 @@ async def ydoc_document_join(sid, data):
             if (
                 user.get("role") != "admin"
                 and user.get("id") != note.user_id
-                and not has_access(
-                    user.get("id"), type="read", access_control=note.access_control
+                and not AccessGrants.has_access(
+                    user_id=user.get("id"),
+                    resource_type="note",
+                    resource_id=note.id,
+                    permission="read",
                 )
             ):
                 log.error(
@@ -531,8 +598,11 @@ async def document_save_handler(document_id, data, user):
         if (
             user.get("role") != "admin"
             and user.get("id") != note.user_id
-            and not has_access(
-                user.get("id"), type="read", access_control=note.access_control
+            and not AccessGrants.has_access(
+                user_id=user.get("id"),
+                resource_type="note",
+                resource_id=note.id,
+                permission="read",
             )
         ):
             log.error(f"User {user.get('id')} does not have access to note {note_id}")
@@ -684,6 +754,17 @@ async def disconnect(sid):
     if sid in SESSION_POOL:
         user = SESSION_POOL[sid]
         del SESSION_POOL[sid]
+
+        # Clean up USAGE_POOL entries for this session
+        for model_id in list(USAGE_POOL.keys()):
+            connections = USAGE_POOL.get(model_id)
+            if connections and sid in connections:
+                del connections[sid]
+                if not connections:
+                    del USAGE_POOL[model_id]
+                else:
+                    USAGE_POOL[model_id] = connections
+
         await YDOC_MANAGER.remove_user_from_all_documents(sid)
     else:
         pass
