@@ -6,6 +6,7 @@ from open_webui.utils.misc import (
 )
 
 from typing import Callable, Optional
+import copy
 import json
 
 
@@ -286,6 +287,13 @@ def convert_payload_openai_to_ollama(openai_payload: dict) -> dict:
     Returns:
         dict: A modified payload compatible with the Ollama API.
     """
+    # Shallow copy metadata separately (may contain non-picklable objects)
+    metadata = openai_payload.get("metadata")
+    openai_payload = copy.deepcopy(
+        {k: v for k, v in openai_payload.items() if k != "metadata"}
+    )
+    if metadata is not None:
+        openai_payload["metadata"] = dict(metadata)
     ollama_payload = {}
 
     # Mapping basic model and message details
@@ -297,6 +305,10 @@ def convert_payload_openai_to_ollama(openai_payload: dict) -> dict:
     if "tools" in openai_payload:
         ollama_payload["tools"] = openai_payload["tools"]
 
+    if "max_tokens" in openai_payload:
+        ollama_payload["num_predict"] = openai_payload["max_tokens"]
+        del openai_payload["max_tokens"]
+
     # If there are advanced parameters in the payload, format them in Ollama's options field
     if openai_payload.get("options"):
         ollama_payload["options"] = openai_payload["options"]
@@ -385,3 +397,29 @@ def convert_embedding_payload_openai_to_ollama(openai_payload: dict) -> dict:
             ollama_payload[optional_key] = openai_payload[optional_key]
 
     return ollama_payload
+
+
+def convert_embed_payload_openai_to_ollama(openai_payload: dict) -> dict:
+    """
+    Convert an embeddings request payload from OpenAI format to Ollama's
+    /api/embed format, which supports batch input natively.
+
+    Args:
+        openai_payload (dict): The original payload designed for OpenAI API usage.
+            Expected keys: "model", "input" (str or list[str]).
+
+    Returns:
+        dict: A payload compatible with the Ollama /api/embed endpoint.
+    """
+    ollama_payload = {"model": openai_payload.get("model")}
+    input_value = openai_payload.get("input")
+
+    # /api/embed accepts 'input' as a string or list of strings directly
+    ollama_payload["input"] = input_value
+
+    # Optionally forward other fields if present
+    for optional_key in ("truncate", "options", "keep_alive"):
+        if optional_key in openai_payload:
+            ollama_payload[optional_key] = openai_payload[optional_key]
+
+    return ollama_payload
