@@ -37,11 +37,49 @@ from langchain_core.utils.function_calling import (
 from open_webui.utils.misc import is_string_allowed
 from open_webui.models.tools import Tools
 from open_webui.models.users import UserModel
+from open_webui.models.groups import Groups
+from open_webui.models.access_grants import AccessGrants
 from open_webui.utils.plugin import load_tool_module_by_id
+from open_webui.utils.access_control import has_access
+from open_webui.config import BYPASS_ADMIN_ACCESS_CONTROL
 from open_webui.env import (
     AIOHTTP_CLIENT_TIMEOUT,
     AIOHTTP_CLIENT_TIMEOUT_TOOL_SERVER_DATA,
     AIOHTTP_CLIENT_SESSION_TOOL_SERVER_SSL,
+    ENABLE_FORWARD_USER_INFO_HEADERS,
+    FORWARD_SESSION_INFO_HEADER_CHAT_ID,
+    FORWARD_SESSION_INFO_HEADER_MESSAGE_ID,
+)
+from open_webui.utils.headers import include_user_info_headers
+from open_webui.tools.builtin import (
+    search_web,
+    fetch_url,
+    generate_image,
+    edit_image,
+    execute_code,
+    search_memories,
+    add_memory,
+    replace_memory_content,
+    get_current_timestamp,
+    calculate_timestamp,
+    search_notes,
+    search_chats,
+    search_channels,
+    search_channel_messages,
+    view_note,
+    view_chat,
+    view_channel_message,
+    view_channel_thread,
+    replace_note_content,
+    write_note,
+    list_knowledge_bases,
+    search_knowledge_bases,
+    query_knowledge_bases,
+    search_knowledge_files,
+    query_knowledge_files,
+    view_file,
+    view_knowledge_file,
+    view_skill,
 )
 
 import copy
@@ -104,146 +142,48 @@ def get_updated_tool_function(function: Callable, extra_params: dict):
     return function
 
 
+def has_tool_server_access(
+    user: UserModel, server_connection: dict, user_group_ids: set = None
+) -> bool:
+    """Check if user has access to a tool server (MCP or OpenAPI)."""
+    if user.role == "admin" and BYPASS_ADMIN_ACCESS_CONTROL:
+        return True
+
+    if user_group_ids is None:
+        user_group_ids = {group.id for group in Groups.get_groups_by_member_id(user.id)}
+
+    server_config = server_connection.get("config", {})
+    access_grants = server_config.get("access_grants", [])
+    return has_access(user.id, "read", access_grants, user_group_ids)
+
+
 async def get_tools(
     request: Request, tool_ids: list[str], user: UserModel, extra_params: dict
 ) -> dict[str, dict]:
+    """Load tools for the given tool_ids, checking access control."""
     tools_dict = {}
 
+    # Get user's group memberships for access control checks
+    user_group_ids = {group.id for group in Groups.get_groups_by_member_id(user.id)}
+
     for tool_id in tool_ids:
         tool = Tools.get_tool_by_id(tool_id)
-        if tool is None:
-
-            if tool_id.startswith("server:"):
-                splits = tool_id.split(":")
-
-                if len(splits) == 2:
-                    type = "openapi"
-                    server_id = splits[1]
-                elif len(splits) == 3:
-                    type = splits[1]
-                    server_id = splits[2]
-
-                server_id_splits = server_id.split("|")
-                if len(server_id_splits) == 2:
-                    server_id = server_id_splits[0]
-                    function_names = server_id_splits[1].split(",")
-
-                if type == "openapi":
-
-                    tool_server_data = None
-                    for server in await get_tool_servers(request):
-                        if server["id"] == server_id:
-                            tool_server_data = server
-                            break
-
-                    if tool_server_data is None:
-                        log.warning(f"Tool server data not found for {server_id}")
-                        continue
-
-                    tool_server_idx = tool_server_data.get("idx", 0)
-                    tool_server_connection = (
-                        request.app.state.config.TOOL_SERVER_CONNECTIONS[
-                            tool_server_idx
-                        ]
-                    )
-
-                    specs = tool_server_data.get("specs", [])
-                    function_name_filter_list = tool_server_connection.get(
-                        "config", {}
-                    ).get("function_name_filter_list", "")
-
-                    if isinstance(function_name_filter_list, str):
-                        function_name_filter_list = function_name_filter_list.split(",")
-
-                    for spec in specs:
-                        function_name = spec["name"]
-                        if function_name_filter_list:
-                            if not is_string_allowed(
-                                function_name, function_name_filter_list
-                            ):
-                                # Skip this function
-                                continue
-
-                        auth_type = tool_server_connection.get("auth_type", "bearer")
-
-                        cookies = {}
-                        headers = {
-                            "Content-Type": "application/json",
-                        }
-
-                        if auth_type == "bearer":
-                            headers["Authorization"] = (
-                                f"Bearer {tool_server_connection.get('key', '')}"
-                            )
-                        elif auth_type == "none":
-                            # No authentication
-                            pass
-                        elif auth_type == "session":
-                            cookies = request.cookies
-                            headers["Authorization"] = (
-                                f"Bearer {request.state.token.credentials}"
-                            )
-                        elif auth_type == "system_oauth":
-                            cookies = request.cookies
-                            oauth_token = extra_params.get("__oauth_token__", None)
-                            if oauth_token:
-                                headers["Authorization"] = (
-                                    f"Bearer {oauth_token.get('access_token', '')}"
-                                )
-
-                        connection_headers = tool_server_connection.get("headers", None)
-                        if connection_headers and isinstance(connection_headers, dict):
-                            for key, value in connection_headers.items():
-                                headers[key] = value
-
-                        def make_tool_function(
-                            function_name, tool_server_data, headers
-                        ):
-                            async def tool_function(**kwargs):
-                                return await execute_tool_server(
-                                    url=tool_server_data["url"],
-                                    headers=headers,
-                                    cookies=cookies,
-                                    name=function_name,
-                                    params=kwargs,
-                                    server_data=tool_server_data,
-                                )
-
-                            return tool_function
-
-                        tool_function = make_tool_function(
-                            function_name, tool_server_data, headers
-                        )
-
-                        callable = get_async_tool_function_and_apply_extra_params(
-                            tool_function,
-                            {},
-                        )
-
-                        tool_dict = {
-                            "tool_id": tool_id,
-                            "callable": callable,
-                            "spec": spec,
-                            # Misc info
-                            "type": "external",
-                        }
-
-                        # Handle function name collisions
-                        while function_name in tools_dict:
-                            log.warning(
-                                f"Tool {function_name} already exists in another tools!"
-                            )
-                            # Prepend server ID to function name
-                            function_name = f"{server_id}_{function_name}"
-
-                        tools_dict[function_name] = tool_dict
-
-                else:
-                    continue
-
-            else:
+        if tool:
+            # Check access control for local tools
+            if (
+                not (user.role == "admin" and BYPASS_ADMIN_ACCESS_CONTROL)
+                and tool.user_id != user.id
+                and not AccessGrants.has_access(
+                    user_id=user.id,
+                    resource_type="tool",
+                    resource_id=tool.id,
+                    permission="read",
+                    user_group_ids=user_group_ids,
+                )
+            ):
+                log.warning(f"Access denied to tool {tool_id} for user {user.id}")
                 continue
-        else:
+
             module = request.app.state.TOOLS.get(tool_id, None)
             if module is None:
                 module, _ = load_tool_module_by_id(tool_id)
@@ -316,6 +256,307 @@ async def get_tools(
                     function_name = f"{tool_id}_{function_name}"
 
                 tools_dict[function_name] = tool_dict
+        else:
+            if tool_id.startswith("server:"):
+                splits = tool_id.split(":")
+
+                if len(splits) == 2:
+                    type = "openapi"
+                    server_id = splits[1]
+                elif len(splits) == 3:
+                    type = splits[1]
+                    server_id = splits[2]
+
+                server_id_splits = server_id.split("|")
+                if len(server_id_splits) == 2:
+                    server_id = server_id_splits[0]
+                    function_names = server_id_splits[1].split(",")
+
+                if type == "openapi":
+
+                    tool_server_data = None
+                    for server in await get_tool_servers(request):
+                        if server["id"] == server_id:
+                            tool_server_data = server
+                            break
+
+                    if tool_server_data is None:
+                        log.warning(f"Tool server data not found for {server_id}")
+                        continue
+
+                    tool_server_idx = tool_server_data.get("idx", 0)
+                    tool_server_connection = (
+                        request.app.state.config.TOOL_SERVER_CONNECTIONS[
+                            tool_server_idx
+                        ]
+                    )
+
+                    # Check access control for tool server
+                    if not has_tool_server_access(
+                        user, tool_server_connection, user_group_ids
+                    ):
+                        log.warning(
+                            f"Access denied to tool server {server_id} for user {user.id}"
+                        )
+                        continue
+
+                    specs = tool_server_data.get("specs", [])
+                    function_name_filter_list = tool_server_connection.get(
+                        "config", {}
+                    ).get("function_name_filter_list", "")
+
+                    if isinstance(function_name_filter_list, str):
+                        function_name_filter_list = function_name_filter_list.split(",")
+
+                    for spec in specs:
+                        function_name = spec["name"]
+                        if function_name_filter_list:
+                            if not is_string_allowed(
+                                function_name, function_name_filter_list
+                            ):
+                                # Skip this function
+                                continue
+
+                        auth_type = tool_server_connection.get("auth_type", "bearer")
+
+                        cookies = {}
+                        headers = {
+                            "Content-Type": "application/json",
+                        }
+
+                        if auth_type == "bearer":
+                            headers["Authorization"] = (
+                                f"Bearer {tool_server_connection.get('key', '')}"
+                            )
+                        elif auth_type == "none":
+                            # No authentication
+                            pass
+                        elif auth_type == "session":
+                            cookies = request.cookies
+                            headers["Authorization"] = (
+                                f"Bearer {request.state.token.credentials}"
+                            )
+                        elif auth_type == "system_oauth":
+                            cookies = request.cookies
+                            oauth_token = extra_params.get("__oauth_token__", None)
+                            if oauth_token:
+                                headers["Authorization"] = (
+                                    f"Bearer {oauth_token.get('access_token', '')}"
+                                )
+
+                        connection_headers = tool_server_connection.get("headers", None)
+                        if connection_headers and isinstance(connection_headers, dict):
+                            for key, value in connection_headers.items():
+                                headers[key] = value
+
+                        # Add user info headers if enabled
+                        if ENABLE_FORWARD_USER_INFO_HEADERS and user:
+                            headers = include_user_info_headers(headers, user)
+                            metadata = extra_params.get("__metadata__", {})
+                            if metadata and metadata.get("chat_id"):
+                                headers[FORWARD_SESSION_INFO_HEADER_CHAT_ID] = (
+                                    metadata.get("chat_id")
+                                )
+                            if metadata and metadata.get("message_id"):
+                                headers[FORWARD_SESSION_INFO_HEADER_MESSAGE_ID] = (
+                                    metadata.get("message_id")
+                                )
+
+                        def make_tool_function(
+                            function_name, tool_server_data, headers
+                        ):
+                            async def tool_function(**kwargs):
+                                return await execute_tool_server(
+                                    url=tool_server_data["url"],
+                                    headers=headers,
+                                    cookies=cookies,
+                                    name=function_name,
+                                    params=kwargs,
+                                    server_data=tool_server_data,
+                                )
+
+                            return tool_function
+
+                        tool_function = make_tool_function(
+                            function_name, tool_server_data, headers
+                        )
+
+                        callable = get_async_tool_function_and_apply_extra_params(
+                            tool_function,
+                            {},
+                        )
+
+                        tool_dict = {
+                            "tool_id": tool_id,
+                            "callable": callable,
+                            "spec": spec,
+                            # Misc info
+                            "type": "external",
+                        }
+
+                        # Handle function name collisions
+                        while function_name in tools_dict:
+                            log.warning(
+                                f"Tool {function_name} already exists in another tools!"
+                            )
+                            # Prepend server ID to function name
+                            function_name = f"{server_id}_{function_name}"
+
+                        tools_dict[function_name] = tool_dict
+
+                else:
+                    continue
+
+    return tools_dict
+
+
+def get_builtin_tools(
+    request: Request, extra_params: dict, features: dict = None, model: dict = None
+) -> dict[str, dict]:
+    """
+    Get built-in tools for native function calling.
+    Only returns tools when BOTH the global config is enabled AND the model capability allows it.
+    """
+    tools_dict = {}
+    builtin_functions = []
+    features = features or {}
+    model = model or {}
+
+    # Helper to get model capabilities (defaults to True if not specified)
+    def get_model_capability(name: str, default: bool = True) -> bool:
+        return (model.get("info", {}).get("meta", {}).get("capabilities") or {}).get(
+            name, default
+        )
+
+    # Helper to check if a builtin tool category is enabled via meta.builtinTools
+    # Defaults to True if not specified (backward compatible)
+    def is_builtin_tool_enabled(category: str) -> bool:
+        builtin_tools = model.get("info", {}).get("meta", {}).get("builtinTools", {})
+        return builtin_tools.get(category, True)
+
+    # Time utilities - available for date calculations
+    if is_builtin_tool_enabled("time"):
+        builtin_functions.extend([get_current_timestamp, calculate_timestamp])
+
+    # Knowledge base tools - conditional injection based on model knowledge
+    # If model has attached knowledge (any type), only provide query_knowledge_files
+    # Otherwise, provide all KB browsing tools
+    model_knowledge = model.get("info", {}).get("meta", {}).get("knowledge", [])
+    if is_builtin_tool_enabled("knowledge"):
+        if model_knowledge:
+            # Model has attached knowledge - only allow semantic search within it
+            builtin_functions.append(query_knowledge_files)
+
+            knowledge_types = {item.get("type") for item in model_knowledge}
+            if "file" in knowledge_types or "collection" in knowledge_types:
+                builtin_functions.append(view_file)
+            if "note" in knowledge_types:
+                builtin_functions.append(view_note)
+        else:
+            # No model knowledge - allow full KB browsing
+            builtin_functions.extend(
+                [
+                    list_knowledge_bases,
+                    search_knowledge_bases,
+                    query_knowledge_bases,
+                    search_knowledge_files,
+                    query_knowledge_files,
+                    view_knowledge_file,
+                ]
+            )
+
+    # Chats tools - search and fetch user's chat history
+    if is_builtin_tool_enabled("chats"):
+        builtin_functions.extend([search_chats, view_chat])
+
+    # Add memory tools if builtin category enabled AND enabled for this chat
+    if is_builtin_tool_enabled("memory") and features.get("memory"):
+        builtin_functions.extend([search_memories, add_memory, replace_memory_content])
+
+    # Add web search tools if builtin category enabled AND enabled globally AND model has web_search capability
+    if (
+        is_builtin_tool_enabled("web_search")
+        and getattr(request.app.state.config, "ENABLE_WEB_SEARCH", False)
+        and get_model_capability("web_search")
+        and features.get("web_search")
+    ):
+        builtin_functions.extend([search_web, fetch_url])
+
+    # Add image generation/edit tools if builtin category enabled AND enabled globally AND model has image_generation capability
+    if (
+        is_builtin_tool_enabled("image_generation")
+        and getattr(request.app.state.config, "ENABLE_IMAGE_GENERATION", False)
+        and get_model_capability("image_generation")
+        and features.get("image_generation")
+    ):
+        builtin_functions.append(generate_image)
+    if (
+        is_builtin_tool_enabled("image_generation")
+        and getattr(request.app.state.config, "ENABLE_IMAGE_EDIT", False)
+        and get_model_capability("image_generation")
+        and features.get("image_generation")
+    ):
+        builtin_functions.append(edit_image)
+
+    # Add code interpreter tool if builtin category enabled AND enabled globally AND model has code_interpreter capability
+    if (
+        is_builtin_tool_enabled("code_interpreter")
+        and getattr(request.app.state.config, "ENABLE_CODE_INTERPRETER", True)
+        and get_model_capability("code_interpreter")
+        and features.get("code_interpreter")
+    ):
+        builtin_functions.append(execute_code)
+
+    # Notes tools - search, view, create, and update user's notes (if builtin category enabled AND notes enabled globally)
+    if is_builtin_tool_enabled("notes") and getattr(
+        request.app.state.config, "ENABLE_NOTES", False
+    ):
+        builtin_functions.extend(
+            [search_notes, view_note, write_note, replace_note_content]
+        )
+
+    # Channels tools - search channels and messages (if builtin category enabled AND channels enabled globally)
+    if is_builtin_tool_enabled("channels") and getattr(
+        request.app.state.config, "ENABLE_CHANNELS", False
+    ):
+        builtin_functions.extend(
+            [
+                search_channels,
+                search_channel_messages,
+                view_channel_thread,
+                view_channel_message,
+            ]
+        )
+
+    # Skills tools - view_skill allows model to load full skill instructions on demand
+    if extra_params.get("__skill_ids__"):
+        builtin_functions.append(view_skill)
+
+    for func in builtin_functions:
+        callable = get_async_tool_function_and_apply_extra_params(
+            func,
+            {
+                "__request__": request,
+                "__user__": extra_params.get("__user__", {}),
+                "__event_emitter__": extra_params.get("__event_emitter__"),
+                "__event_call__": extra_params.get("__event_call__"),
+                "__metadata__": extra_params.get("__metadata__"),
+                "__chat_id__": extra_params.get("__chat_id__"),
+                "__message_id__": extra_params.get("__message_id__"),
+                "__model_knowledge__": model_knowledge,
+            },
+        )
+
+        # Generate spec from function
+        pydantic_model = convert_function_to_pydantic_model(func)
+        spec = convert_pydantic_model_to_openai_function_spec(pydantic_model)
+
+        tools_dict[func.__name__] = {
+            "tool_id": f"builtin:{func.__name__}",
+            "callable": callable,
+            "spec": spec,
+            "type": "builtin",
+        }
 
     return tools_dict
 
@@ -500,9 +741,10 @@ def convert_openapi_to_tool_payload(openapi_spec):
                     "parameters": {"type": "object", "properties": {}, "required": []},
                 }
 
-                # Extract path and query parameters
                 for param in operation.get("parameters", []):
-                    param_name = param["name"]
+                    param_name = param.get("name")
+                    if not param_name:
+                        continue
                     param_schema = param.get("schema", {})
                     description = param_schema.get("description", "")
                     if not description:
@@ -774,8 +1016,10 @@ async def execute_tool_server(
         body_params = {}
 
         for param in operation.get("parameters", []):
-            param_name = param["name"]
-            param_in = param["in"]
+            param_name = param.get("name")
+            if not param_name:
+                continue
+            param_in = param.get("in")
             if param_name in params:
                 if param_in == "path":
                     path_params[param_name] = params[param_name]
