@@ -65,15 +65,7 @@ export const unarchiveAllChats = async (token: string) => {
 	return res;
 };
 
-export const importChat = async (
-	token: string,
-	chat: object,
-	meta: object | null,
-	pinned?: boolean,
-	folderId?: string | null,
-	createdAt: number | null = null,
-	updatedAt: number | null = null
-) => {
+export const importChats = async (token: string, chats: object[]) => {
 	let error = null;
 
 	const res = await fetch(`${WEBUI_API_BASE_URL}/chats/import`, {
@@ -84,12 +76,7 @@ export const importChat = async (
 			authorization: `Bearer ${token}`
 		},
 		body: JSON.stringify({
-			chat: chat,
-			meta: meta ?? {},
-			pinned: pinned,
-			folder_id: folderId,
-			created_at: createdAt ?? null,
-			updated_at: updatedAt ?? null
+			chats
 		})
 	})
 		.then(async (res) => {
@@ -112,6 +99,7 @@ export const importChat = async (
 export const getChatList = async (
 	token: string = '',
 	page: number | null = null,
+	include_pinned: boolean = false,
 	include_folders: boolean = false
 ) => {
 	let error = null;
@@ -125,6 +113,10 @@ export const getChatList = async (
 		searchParams.append('include_folders', 'true');
 	}
 
+	if (include_pinned) {
+		searchParams.append('include_pinned', 'true');
+	}
+
 	const res = await fetch(`${WEBUI_API_BASE_URL}/chats/?${searchParams.toString()}`, {
 		method: 'GET',
 		headers: {
@@ -150,6 +142,10 @@ export const getChatList = async (
 		throw error;
 	}
 
+	if (!res) {
+		return [];
+	}
+
 	return res.map((chat) => ({
 		...chat,
 		time_range: getTimeRange(chat.updated_at)
@@ -259,6 +255,51 @@ export const getArchivedChatList = async (
 	}));
 };
 
+export const getSharedChatList = async (token: string = '', page: number = 1, filter?: object) => {
+	let error = null;
+
+	const searchParams = new URLSearchParams();
+	searchParams.append('page', `${page}`);
+
+	if (filter) {
+		Object.entries(filter).forEach(([key, value]) => {
+			if (value !== undefined && value !== null) {
+				searchParams.append(key, value.toString());
+			}
+		});
+	}
+
+	const res = await fetch(`${WEBUI_API_BASE_URL}/chats/shared?${searchParams.toString()}`, {
+		method: 'GET',
+		headers: {
+			Accept: 'application/json',
+			'Content-Type': 'application/json',
+			...(token && { authorization: `Bearer ${token}` })
+		}
+	})
+		.then(async (res) => {
+			if (!res.ok) throw await res.json();
+			return res.json();
+		})
+		.then((json) => {
+			return json;
+		})
+		.catch((err) => {
+			error = err;
+			console.error(err);
+			return null;
+		});
+
+	if (error) {
+		throw error;
+	}
+
+	return res.map((chat) => ({
+		...chat,
+		time_range: getTimeRange(chat.updated_at)
+	}));
+};
+
 export const getAllChats = async (token: string) => {
 	let error = null;
 
@@ -1174,3 +1215,103 @@ export const archiveAllChats = async (token: string) => {
 
 	return res;
 };
+export const exportChatStats = async (token: string, page: number = 1, params: object = {}) => {
+	let error = null;
+
+	const searchParams = new URLSearchParams();
+	searchParams.append('page', `${page}`);
+
+	if (params) {
+		for (const [key, value] of Object.entries(params)) {
+			searchParams.append(key, `${value}`);
+		}
+	}
+
+	const res = await fetch(`${WEBUI_API_BASE_URL}/chats/stats/export?${searchParams.toString()}`, {
+		method: 'GET',
+		headers: {
+			Accept: 'application/json',
+			'Content-Type': 'application/json',
+			...(token && { authorization: `Bearer ${token}` })
+		}
+	})
+		.then(async (res) => {
+			if (!res.ok) throw await res.json();
+			return res.json();
+		})
+		.then((json) => {
+			return json;
+		})
+		.catch((err) => {
+			error = err;
+			console.error(err);
+			return null;
+		});
+
+	if (error) {
+		throw error;
+	}
+
+	return res;
+};
+
+export const exportSingleChatStats = async (token: string, chatId: string) => {
+	let error = null;
+
+	const res = await fetch(`${WEBUI_API_BASE_URL}/chats/stats/export/${chatId}`, {
+		method: 'GET',
+		headers: {
+			Accept: 'application/json',
+			'Content-Type': 'application/json',
+			...(token && { authorization: `Bearer ${token}` })
+		}
+	})
+		.then(async (res) => {
+			if (!res.ok) throw await res.json();
+			return res.json();
+		})
+		.then((json) => {
+			return json;
+		})
+		.catch((err) => {
+			error = err;
+			console.error(err);
+			return null;
+		});
+
+	if (error) {
+		throw error;
+	}
+
+	return res;
+};
+
+export const downloadChatStats = async (
+	token: string = '',
+	updated_at: number | null = null
+): Promise<[Response | null, AbortController]> => {
+	const controller = new AbortController();
+	let error = null;
+
+	let url = `${WEBUI_API_BASE_URL}/chats/stats/export?stream=true`;
+	if (updated_at) url += `&updated_at=${updated_at}`;
+
+	const res = await fetch(url, {
+		signal: controller.signal,
+		method: 'GET',
+		headers: {
+			'Content-Type': 'application/json',
+			Authorization: `Bearer ${token}`
+		}
+	}).catch((err) => {
+		console.error(err);
+		error = err;
+		return null;
+	});
+
+	if (error) {
+		throw error;
+	}
+
+	return [res, controller];
+};
